package main

import (
	"encoding/json"
	"fmt"
	"log/slog"

	// "github.com/gofiber/fiber/v2"
	// "github.com/gofiber/fiber/v2/middleware/cors"
	// "github.com/gofiber/swagger"

	"github.com/gofiber/fiber/v2"

	//"github.com/gofiber/swagger"

	_ "code.wuban.net.cn/odysseus/twitter_syncer/docs"
	// docs are generated by Swag CLI, you have to import them.
	// replace with your own docs folder, usually "github.com/username/reponame/docs"
	//_ "github.com/gofiber/swagger/example/docs"
)

type Res struct {
	Code int64  `json:"code"`
	Msg  string `json:"msg"`
}

type VerifyRes struct {
	Code int64  `json:"code"`
	Msg  string `json:"msg"`
	Data struct {
		Ok bool `json:"ok"`
	} `json:"data"`
}

/*

{
  "user_id": "1570057485914087429",
  "project": "OnlyDD_D",
  "task_type": "follow",
  "start": true
}

*/

type AddTaskReq struct {
	// AddOrStop bool
	User     string `json:"user"`
	TaskType string `json:"task_type"`
	TaskId   string `json:"task_id"`
}

var taskIn chan<- taskInterface

func TaskAdd(c *fiber.Ctx) error {

	//fmt.Println(string(c.Request().Body()))

	slog.Info(c.Route().Path, "body", string(c.Request().Body()))

	req := AddTaskReq{}

	if err := json.Unmarshal(c.Request().Body(), &req); err != nil {
		slog.Error("json.Unmarshal(c.Request().Body(), &req)", "err", err.Error())
		return c.JSON(Res{
			Code: 500,
			Msg:  err.Error(),
		})
	}

	slog.Info(c.Route().Path, "user", req.User, "TaskType", req.TaskType, "TaskId", req.TaskId)

	if req.TaskType == "" || req.TaskId == "" {
		return c.JSON(Res{
			Code: 500,
			Msg:  "must provide TaskId and TaskType",
		})
	}

	var task taskInterface

	if req.TaskType == FollowType {
		task = NewFollowTask(req.TaskId, req.User, req.TaskType)
	}

	if req.TaskType == RetweetType {
		task = NewRetweetTask(req.TaskId, req.User, req.TaskType)
	}

	taskIn <- task

	//Todo
	// 校验任务 条件是否存在；

	// req.AddOrStop = true
	err := AddTaskInsertOrUpdate(req)

	//res, _, err := client.From("twitter_task").Insert(req, true, "", "representation", "").Execute()

	if err != nil {

		slog.Error("twitter_syncer insert", "err", err.Error())
		return c.JSON(Res{
			Code: 500,
			Msg:  err.Error(),
		})
	}

	//slog.Info("twitter_syncer insert", "res", string(res))

	return c.JSON(Res{
		Code: 200,
	})
}

func TaskStop(c *fiber.Ctx) error {

	slog.Info(c.Route().Path, "body", string(c.Request().Body()))

	req := AddTaskReq{}

	if err := json.Unmarshal(c.Request().Body(), &req); err != nil {
		slog.Error("json.Unmarshal(c.Request().Body(), &req)", "err", err.Error())
		return c.JSON(Res{
			Code: 500,
			Msg:  err.Error(),
		})
	}

	slog.Info(c.Route().Path, "user", req.User, "TaskType", req.TaskType, "TaskId", req.TaskId)

	if req.TaskType == "" || req.TaskId == "" {
		return c.JSON(Res{
			Code: 500,
			Msg:  "must provide TaskId and TaskType",
		})
	}

	err := StopTaskUpdate(req)

	if err != nil {

		slog.Error("twitter_syncer stop", "err", err.Error())
		return c.JSON(Res{
			Code: 500,
			Msg:  err.Error(),
		})
	}

	return c.JSON(Res{
		Code: 200,
	})

}

func VerifyFollower(c *fiber.Ctx) error {

	userId := c.Query("user_id")
	followerId := c.Query("follower_id")

	if len(userId) == 0 || len(followerId) == 0 {
		slog.Error("VerifyFollower", "userId", userId, "followerId", followerId)
		return c.JSON(Res{
			Code: 500,
			Msg:  fmt.Sprintf("must provide userId [%v] and followerId [%v]", userId, followerId),
		})
	}

	followerUserName := c.Query("follower_username")

	slog.Info(c.Route().Path, "user_id", userId, "followerId", followerId, "followerUserName", followerUserName)

	ok, err := VerifyFollowerInDb(userId, followerId)

	if err != nil {
		slog.Error("VerifyFollowerInDb", "userId", userId, "followerId", followerId, "err", err.Error())
		return c.JSON(Res{
			Code: 500,
			Msg:  fmt.Sprintf("VerifyFollowerInDb  userId   %v followerId %v  err %v", userId, followerId, err.Error()),
		})
	}

	return c.JSON(VerifyRes{
		Code: 200,
		Data: struct {
			Ok bool "json:\"ok\""
		}{
			Ok: ok,
		},
	})

}

func VerifyRetweeter(c *fiber.Ctx) error {

	tweetId := c.Query("tweet_id")
	retweeterId := c.Query("retweeter_id")

	if len(tweetId) == 0 || len(retweeterId) == 0 {
		slog.Error("VerifyFollower", "tweetId", tweetId, "retweeterId", retweeterId)
		return c.JSON(Res{
			Code: 500,
			Msg:  fmt.Sprintf("must provide tweetId [%v] and retweeterId [%v]", tweetId, retweeterId),
		})
	}

	slog.Info(c.Route().Path, "tweetId", tweetId, "retweeterId", retweeterId)

	ok, err := VerifyRetweeterInDb(tweetId, retweeterId)

	if err != nil {
		slog.Error("VerifyRetweeter", "tweetId", tweetId, "retweeterId", retweeterId, "err", err.Error())
		return c.JSON(Res{
			Code: 500,
			//Msg:  fmt.Sprint("VerifyFollowerInDb", "tweetId", tweetId, "retweeterId", retweeterId, "err", err.Error()),
			Msg: fmt.Sprintf("VerifyRetweeter  tweetId   %v retweeterId %v  err %v", tweetId, retweeterId, err.Error()),
		})
	}

	return c.JSON(VerifyRes{
		Code: 200,
		Data: struct {
			Ok bool "json:\"ok\""
		}{
			Ok: ok,
		},
	})

}
