package main

import (
	"container/list"
	b64 "encoding/base64"
	"encoding/json"
	"fmt"
	"log/slog"
	"strings"
	"time"

	twitterscraper "github.com/imperatrona/twitter-scraper"
)

func newSync(done <-chan interface{}) (chan<- taskInterface, error) {

	//done := make(<-chan interface{})

	connStream := make(chan taskInterface, 1000)

	tasks, err := GetTasksIdx()
	if err != nil {
		return nil, err
	}
	for _, task := range tasks {
		connStream <- task
	}

	idxStream, listStream := TaskIdx(done, connStream)

	usersStream := BackListToQueue(done, listStream)

	if err := InsertOrUpdateUsers(done, usersStream); err != nil {
		return nil, err
	}

	resStream, err := InitResource()
	if err != nil {
		return nil, err
	}

	availRes := AvailableResource(done, resStream)

	taskFetchStream, resTailStream := TaskImplement(done, idxStream, availRes)

	ConnectTailTask(done, connStream, taskFetchStream)

	ConnectTailResource(done, resStream, resTailStream)

	//select {}

	return connStream, nil

}

func TaskIdx(done <-chan interface{}, inStream <-chan taskInterface) (<-chan taskInterface, <-chan TaskIdAndList) {

	outStream := make(chan taskInterface, 1000)

	outListStream := make(chan TaskIdAndList, 1)

	go func() {
		for {
			select {
			case <-done:
				return
			case task := <-inStream:

				slog.Info("TaskIdx", "task.TaskType()", task.TaskType(), "task.ID()", task.ID(), "task.UserName()", task.UserName(), "task.InitIdx()", task.InitIdx())

				if !task.InitIdx() {

					list, ok := task.UpdateIdx()

					if ok {
						select {
						case <-done:
							return
						case outListStream <- TaskIdAndList{
							TaskId:   task.ID(),
							List:     list,
							TaskType: task.TaskType(),
						}:
						}

					}
				}

				select {
				case <-done:
					return
				case outStream <- task:
				}

			}
		}
	}()

	return outStream, outListStream
}

type TaskIdAndList struct {
	TaskType string
	TaskId   string
	List     *list.List
}

type TaskIdAndProfiles struct {
	TaskType string
	TaskId   string
	Profiles []Profile
}

func BackListToQueue(done <-chan interface{}, inStream <-chan TaskIdAndList) <-chan TaskIdAndProfiles {

	outStream := make(chan TaskIdAndProfiles, 1)

	go func() {

		for {
			select {
			case <-done:
				return
			case users, ok := <-inStream:
				if ok == false {
					return
				}

				c := 0

				// if c < 100 {
				// 	c = c + 1

				res := make([]Profile, 0, users.List.Len())

				for e := users.List.Front(); e != nil; e = e.Next() {
					if user, ok := e.Value.(Profile); ok {
						//fmt.Printf("The data is a string: %s\n", str)
						res = append(res, user)

						c = c + 1

						if c%100 == 0 {
							fmt.Println("BackListToQueue", "len(inStream)", len(inStream), "len(outStream)", len(outStream), "len(res)", len(res))

							select {
							case <-done:
								return
							case outStream <- TaskIdAndProfiles{
								Profiles: res,
								TaskId:   users.TaskId,
								TaskType: users.TaskType,
							}:

								res = make([]Profile, 0, users.List.Len())
							}

						}
					}
				}

				fmt.Println("BackListToQueue", "len(inStream)", len(inStream), "len(outStream)", len(outStream), "len(res)", len(res))

				select {
				case <-done:
					return
				case outStream <- TaskIdAndProfiles{
					Profiles: res,
					TaskId:   users.TaskId,
					TaskType: users.TaskType,
				}:
				}

				// } else {
				// 	c = 0
				// }

			}
		}
	}()

	return outStream
}

func InsertOrUpdateUsers(done <-chan interface{}, inStream <-chan TaskIdAndProfiles) error {

	// client, err := supabase.NewClient(API_URL, API_KEY, nil)
	// if err != nil {
	// 	return fmt.Errorf("cannot initalize client: %v .", err)
	// }

	go func() {

		for {
			select {
			case <-done:
				return
			case users, ok := <-inStream:
				if ok == false {
					return
				}

				//rows := make([]map[string]string, 0, len(users.Profiles))

				//res, c, err := client.From("followers").Insert(rows, true, "", "representation", "").Execute()

				var res []byte
				var err error

				if users.TaskType == FollowType {
					rows := make([]Follower, 0, len(users.Profiles))

					for _, user := range users.Profiles {

						sDec, _ := b64.StdEncoding.DecodeString(user.UserID)
						userId, _ := strings.CutPrefix(string(sDec), "User:")

						row := Follower{
							Follower: userId,
							UserName: user.Username,
							UserId:   users.TaskId,
						}
						rows = append(rows, row)
					}
					res, _, err = client.From("followers").Insert(rows, true, "", "representation", "").Execute()
				}

				if users.TaskType == RetweetType {

					rows := make([]Retweeter, 0, len(users.Profiles))

					for _, user := range users.Profiles {

						sDec, _ := b64.StdEncoding.DecodeString(user.UserID)
						userId, _ := strings.CutPrefix(string(sDec), "User:")

						row := Retweeter{
							TweetId:           users.TaskId,  //string `json:"tweet_id"`
							RetweeterId:       userId,        //string `json:"retweeter_id"`
							RetweeterUserName: user.Username, //string `json:"retweeter_username"`
							// Follower: userId,
							// UserName: user.Username,
							// UserId:   users.TaskId,
						}
						rows = append(rows, row)
					}

					res, _, err = client.From("retweeters").Insert(rows, true, "", "representation", "").Execute()
				}

				if err != nil {
					slog.Error("insert into followers or retweeters ", err)

					for _, user := range users.Profiles {
						usersAsJson, err := json.Marshal(user)
						if err != nil {
							slog.Error("insert into followers or retweeters json.Marshal", err)
							continue
						}

						sDec, _ := b64.StdEncoding.DecodeString(user.UserID)
						userId, _ := strings.CutPrefix(string(sDec), "User:")

						slog.Error("insert into followers or retweeters error", string(usersAsJson), userId)
					}

				} else {
					slog.Info("insert into followers or retweeters", string(res), err)
				}

				fmt.Println("InsertOrUpdateUsers", "len(inStream)", len(inStream))

			}
		}

	}()

	return nil

}

type ScraperTimer struct {
	Scraper *twitterscraper.Scraper
	Timer   time.Timer
}

func InitResource() (chan ScraperTimer, error) {

	outStream := make(chan ScraperTimer, 1000)

	accounts, err := TwitterAccountFromDB()

	if err != nil {
		return nil, err
	}

	for _, v := range accounts {

		fmt.Println(v.User, v.PassWd)
		scraper, err := InitScraper(v.User, v.PassWd)

		if err != nil {
			return nil, err
		}

		newScraperTimer := ScraperTimer{
			Scraper: scraper,
			Timer:   *time.NewTimer(0),
		}

		//

		outStream <- newScraperTimer

	}

	fmt.Println("twitter init ok")

	return outStream, nil
}

func AvailableResource(done <-chan interface{}, inStream <-chan ScraperTimer) <-chan ScraperTimer {

	outStream := make(chan ScraperTimer, 1000)

	go func() {

		for {
			select {
			case <-done:
				return
			case scraper := <-inStream:
				//TODO
				// 1. 携带web服务器的rate limit 做判断；
				// 2. 校验 是否可用

				slog.Info("AvailableResource------------------------------")

				<-scraper.Timer.C
				scraper.Timer = *time.NewTimer(60 * time.Second)
				//scraper.Timer.Reset(30 * time.Second)
				outStream <- scraper
			}
		}

	}()

	return outStream
}

func TaskImplement(done <-chan interface{}, inTaskStream <-chan taskInterface, inResourceStream <-chan ScraperTimer) (<-chan taskInterface, <-chan ScraperTimer) {

	taskOutStream := make(chan taskInterface, 1000)
	scraperOutStream := make(chan ScraperTimer, 1000)

	go func() {

		for {
			select {
			case <-done:
				return
			case task := <-inTaskStream:

				select {
				case <-done:
					return
				case res := <-inResourceStream:

					slog.Info("TaskImplement", "task.TaskType()", task.TaskType(), "task.ID()", task.ID(), "task.UserName()", task.UserName(), "len(inResourceStream)", len(inResourceStream))

					if err := task.Fetch(res.Scraper); err != nil {
						slog.Error("task.Fetch", "err", err.Error())
					}

					select {
					case <-done:
						return
					case taskOutStream <- task:

					}

					select {
					case <-done:
						return
					case scraperOutStream <- res:

					}
				}
			}
		}
	}()

	return taskOutStream, scraperOutStream
}

func ConnectTailTask(done <-chan interface{}, beginStream chan<- taskInterface, tailStream <-chan taskInterface) {

	go func() {

		for {

			select {
			case <-done:
				return
			case beginStream <- <-tailStream:
			}

		}

	}()

}

func ConnectTailResource(done <-chan interface{}, beginStream chan<- ScraperTimer, tailStream <-chan ScraperTimer) {

	go func() {
		for {
			select {
			case <-done:
				return
			case beginStream <- <-tailStream:
			}
		}
	}()
}
