package main

import (
	"context"
	"fmt"
	"net/http"

	twitter "github.com/g8rswimmer/go-twitter/v2"
)

//https://github.com/michimani/gotwi.git

const (
	OAuthTokenEnvKeyName       = "GOTWI_ACCESS_TOKEN"
	OAuthTokenSecretEnvKeyName = "GOTWI_ACCESS_TOKEN_SECRET"
)

type authorize struct {
	Token string
}

func (a authorize) Add(req *http.Request) {
	req.Header.Add("Authorization", fmt.Sprintf("Bearer %s", a.Token))
}

type Client struct {
	*twitter.Client
}

func NewOAuth2Client() *Client {

	client := &twitter.Client{
		Authorizer: authorize{
			Token: "",
		},
		Client: http.DefaultClient,
		Host:   "https://api.twitter.com",
	}

	return &Client{
		Client: client,
	}
}

func (c *Client) Followers(userId string, next string) ([]*twitter.UserObj, string, *twitter.RateLimit, error) {

	opts := twitter.UserFollowersLookupOpts{
		Expansions:      []twitter.Expansion{twitter.ExpansionPinnedTweetID},
		TweetFields:     []twitter.TweetField{twitter.TweetFieldContextAnnotations},
		PaginationToken: next,
	}

	fmt.Println("Callout to user followers lookup callout")

	userResponse, err := c.Client.UserFollowersLookup(context.Background(), userId, opts)

	if err != nil {
		return nil, "", nil, err
	}

	return userResponse.Raw.Users, userResponse.Meta.NextToken, userResponse.RateLimit, nil
}

func (c *Client) Retweeters(tweetId string, next string) ([]*twitter.UserObj, string, *twitter.RateLimit, error) {

	opts := twitter.UserRetweetLookupOpts{
		Expansions:      []twitter.Expansion{twitter.ExpansionPinnedTweetID},
		PaginationToken: next,
	}

	userResponse, err := c.Client.UserRetweetLookup(context.Background(), tweetId, opts)
	if err != nil {
		return nil, "", nil, err
	}

	return userResponse.Raw.Users, userResponse.Meta.NextToken, userResponse.RateLimit, nil

}

func (c *Client) TweetLikingUsers(tweetId string, next string) ([]*twitter.UserObj, string, *twitter.RateLimit, error) {

	opts := twitter.TweetLikesLookupOpts{
		Expansions:      []twitter.Expansion{twitter.ExpansionPinnedTweetID},
		TweetFields:     []twitter.TweetField{twitter.TweetFieldCreatedAt, twitter.TweetFieldConversationID, twitter.TweetFieldAttachments},
		PaginationToken: next,
	}

	fmt.Println("Callout to tweet like lookup callout")

	tweetResponse, err := c.Client.TweetLikesLookup(context.Background(), tweetId, opts)
	if err != nil {
		return nil, "", nil, err
	}

	return tweetResponse.Raw.Users, tweetResponse.Meta.NextToken, tweetResponse.RateLimit, nil

}
