package main

import (
	b64 "encoding/base64"
	"encoding/json"
	"fmt"
	"log/slog"
	"strings"

	"github.com/supabase-community/postgrest-go"
)

const FollowType = "followers"
const RetweetType = "retweeters"
const TweetLikingUsersType = "tweet_liking_users"

func ListToQueue(done <-chan interface{}, inStream <-chan TaskIdAndList) <-chan TaskIdAndProfiles {

	outStream := make(chan TaskIdAndProfiles, 1)

	go func() {

		for {
			select {
			case <-done:
				return
			case users, ok := <-inStream:
				if ok == false {
					return
				}

				c := 0

				// if c < 100 {
				// 	c = c + 1

				res := make([]Profile, 0, users.List.Len())

				for e := users.List.Front(); e != nil; e = e.Next() {
					if user, ok := e.Value.(Profile); ok {
						//fmt.Printf("The data is a string: %s\n", str)
						res = append(res, user)

						c = c + 1

						if c%100 == 0 {
							fmt.Println("BackListToQueue", "len(inStream)", len(inStream), "len(outStream)", len(outStream), "len(res)", len(res))

							select {
							case <-done:
								return
							case outStream <- TaskIdAndProfiles{
								Profiles: res,
								TaskId:   users.TaskId,
								TaskType: users.TaskType,
							}:

								res = make([]Profile, 0, users.List.Len())
							}

						}
					}
				}

				fmt.Println("BackListToQueue", "len(inStream)", len(inStream), "len(outStream)", len(outStream), "len(res)", len(res))

				select {
				case <-done:
					return
				case outStream <- TaskIdAndProfiles{
					Profiles: res,
					TaskId:   users.TaskId,
					TaskType: users.TaskType,
				}:
				}

				// } else {
				// 	c = 0
				// }

			}
		}
	}()

	return outStream
}

func InsertOrUpdateFinishTask(done <-chan interface{}, inStream <-chan TaskIdAndProfiles) error {

	go func() {

		for {
			select {
			case <-done:
				return
			case users, ok := <-inStream:
				if ok == false {
					return
				}

				var res []byte
				var err error

				//if users.TaskType == FollowType {
				rows := make([]Follower, 0, len(users.Profiles))

				for _, user := range users.Profiles {

					sDec, _ := b64.StdEncoding.DecodeString(user.UserID)
					userId, _ := strings.CutPrefix(string(sDec), "User:")

					row := Follower{
						Follower: userId,
						UserName: user.Username,
						UserId:   users.TaskId,
					}
					rows = append(rows, row)
				}
				res, _, err = client.From(users.TaskType).Insert(rows, true, "", "representation", "").Execute()
				//}

				if err != nil {
					slog.Error("insert into followers or retweeters ", err)

					for _, user := range users.Profiles {
						usersAsJson, err := json.Marshal(user)
						if err != nil {
							slog.Error("insert into followers or retweeters json.Marshal", err)
							continue
						}

						sDec, _ := b64.StdEncoding.DecodeString(user.UserID)
						userId, _ := strings.CutPrefix(string(sDec), "User:")

						slog.Error("insert into followers or retweeters error", string(usersAsJson), userId)
					}

				} else {
					slog.Info("insert into followers or retweeters", string(res), err)
				}

				fmt.Println("InsertOrUpdateUsers", "len(inStream)", len(inStream))

			}
		}

	}()

	return nil

}

type UserTaskIdAndTime struct {
	UserTask
	Id        int    `json:"id"`
	CreatedAt string `json:"created_at"`
}

type UserTask struct {
	//user_id
	TaskId   string `json:"task_id"`
	UserId   string `json:"user_id"`
	UserName string `json:"user_name"`
}

type Task struct {
	// ID       int    `json:"id"`
	User     string `json:"user"`
	TaskType string `json:"task_type"`
	TaskId   string `json:"task_id"`
	Start    bool   `json:"start"`
	Stop     bool   `json:"stop"`
}

func QueryAllTask() ([]Task, error) {

	data, count, err := client.From("tasks").Select("*", "exact", false).
		Eq("start", "true").Neq("stop", "true").
		Execute()

	if err != nil {
		return nil, err
	}

	_ = count

	// fmt.Println(count, string(data))

	res := make([]Task, 0, count)

	if err := json.Unmarshal(data, &res); err != nil {
		return nil, err
	}

	return res, nil

}

func GetTasksIdxWithTaskType() ([]*TaskJob, error) {

	tasks, err := QueryAllTask()

	if err != nil {
		return nil, err
	}

	res := make([]*TaskJob, 0, 10)

	for _, task := range tasks {

		//if task.TaskType == FollowType {

		data, count, err := client.From(task.TaskType).Select("", "user_id", false).
			Eq("user_id", task.TaskId).
			Order("id", &postgrest.OrderOpts{
				Ascending: false,
				// NullsFirst   bool
				// ForeignTable string
			}).Range(0, 10, "").Execute()

		if err != nil {
			slog.Error("select * from followers error", err)
			return nil, err
		}
		_ = count

		slog.Info("idx data", "user id", task.TaskId, "user name", task.User, "idx", data)

		fmt.Println("idx data", string(data))

		userRes := make([]UserTaskIdAndTime, 0, 10)

		if err := json.Unmarshal(data, &userRes); err != nil {
			return nil, err
		}

		taskJob := NewTaskJob(task.TaskId, task.User, task.TaskType)

		taskJob.Idx = userRes

		res = append(res, taskJob)

	}

	return res, nil
}
