package main

import (
	"encoding/json"
	"fmt"
	"log/slog"
	"time"

	twitter "github.com/g8rswimmer/go-twitter/v2"
	"github.com/supabase-community/postgrest-go"
)

type streamIdx struct {
	cli       Client
	idx       []Task
	taskId    string
	toDbQueue chan []UserTask
	// timer  time.Timer
}

func (s *streamIdx) Request(taskType, next string) ([]*twitter.UserObj, string, *twitter.RateLimit, error) {

	switch taskType {
	case FollowType:
		return s.cli.Followers(s.taskId, next)
	case RetweetType:
		return s.cli.Retweeters(s.taskId, next)
	case TweetLikingUsersType:
		return s.cli.TweetLikingUsers(s.taskId, next)
	}

	return nil, "", nil, fmt.Errorf("currently not suport the task type %v", taskType)
}

func (s *streamIdx) Idx() {

}

func (s *streamIdx) Scheduler(timer *time.Timer) {

	for t := range timer.C {
		_ = t
		s.Idx()
		//s.Scheduler(time.NewTimer(10 * time.Second))
	}
}

// func scheduler(tick *time.Ticker) {
// 	for t := range tick.C {
// 		task(t)
// 	}
// }

// func task(t time.Time) {
// 	fmt.Println("hello! printed at ", t)
// }

// func GetFollowTasks() ([]Task, error) {

// 	data, count, err := client.From("tasks").Select("*", "exact", false).
// 		Eq("follow", "true").Neq("follow_stop", "true").
// 		Execute()

// 	if err != nil {
// 		return nil, err
// 	}

// 	res := make([]Task, 0, count)

// 	if err := json.Unmarshal(data, &res); err != nil {
// 		return nil, err
// 	}

// 	return res, nil
// }

// follow, retweet
// const FollowType = "follow"
// const RetweetType = "retweet"
// const FollowType = "follow"

// followers

// const RetweetType = "retweet"

type FollowerId struct {
	Follower
	Id        int    `json:"id"`
	CreatedAt string `json:"created_at"`
}

type Follower struct {
	//user_id
	UserId   string `json:"user_id"`
	Follower string `json:"follower_id"`
	UserName string `json:"follower_username"`
}

type RetweeterId struct {
	Retweeter
	Id        int    `json:"id"`
	CreatedAt string `json:"created_at"`
}

type Retweeter struct {
	TweetId           string `json:"tweet_id"`
	RetweeterId       string `json:"retweeter_id"`
	RetweeterUserName string `json:"retweeter_username"`
}

func GetTasksIdx() ([]taskInterface, error) {

	tasks, err := QueryAllTask()

	if err != nil {
		return nil, err
	}

	res := make([]taskInterface, 0, 10)

	for _, task := range tasks {

		if task.TaskType == FollowType {

			data, count, err := client.From("followers").Select("", "user_id", false).
				Eq("user_id", task.TaskId).
				Order("id", &postgrest.OrderOpts{
					Ascending: false,
					// NullsFirst   bool
					// ForeignTable string
				}).Range(0, 10, "").Execute()

			if err != nil {
				slog.Error("select * from followers error", err)
				return nil, err
			}
			_ = count

			slog.Info("idx data", "user id", task.TaskId, "user name", task.User, "idx", data)

			fmt.Println("idx data", string(data))

			userRes := make([]FollowerId, 0, 10)

			if err := json.Unmarshal(data, &userRes); err != nil {
				return nil, err
			}

			followTask := NewFollowTask(task.TaskId, task.User, task.TaskType)

			followTask.Idx = userRes

			// if len(userRes) == 0 {
			// 	followTask.Init = true
			// }

			res = append(res, followTask)
		}

		if task.TaskType == RetweetType {

			data, count, err := client.From("retweeters").Select("", "tweet_id", false).
				Eq("tweet_id", task.TaskId).
				Order("id", &postgrest.OrderOpts{
					Ascending: false,
					// NullsFirst   bool
					// ForeignTable string
				}).Range(0, 10, "").Execute()

			if err != nil {
				slog.Error("select * from retweeters error", err)
				return nil, err
			}
			_ = count

			slog.Info("idx data", "tweet id", task.TaskId, "user name", task.User, "idx", data)

			fmt.Println("idx data", string(data))

			userRes := make([]RetweeterId, 0, 10)

			if err := json.Unmarshal(data, &userRes); err != nil {
				return nil, err
			}

			retweetTask := NewRetweetTask(task.TaskId, task.User, task.TaskType)

			//followTask := NewFollowTask(task.TaskId, task.User, task.TaskType)

			retweetTask.Idx = userRes

			// if len(userRes) == 0 {
			// 	followTask.Init = true
			// }

			res = append(res, retweetTask)

		}
	}

	return res, nil
}

// func FollowersToBackList(done <-chan interface{}, idxss [][]FollowerId) (<-chan *list.List, error) {

// 	scraper := twitterscraper.New()

// 	//err := scraper.Login("Wade_Leeeee", "923881393time")
// 	err := scraper.Login("wuban358369", "123456789T")
// 	if err != nil {
// 		fmt.Println("scraper", err.Error())
// 		return nil, err
// 	}

// 	/*
// 					wuban01@gmail.com

// 					wuban01
// 					1234567890Wuban

// 					1234567890Wuban

// 					// 2346
// 			980b
// 			641a
// 			774d
// 			39fb
// 			9425
// 			c6c5
// 			3140
// 			b469
// 			0b93
// 			d1c6
// 			6101
// 			ef6f
// 			9fb6
// 			ae94
// 			ee6f

// 				//wuban001

// 		wuban01@tutamail.com

// 		1234567890Wuban

// 	*/

// 	scraper3 := twitterscraper.New()

// 	//err = scraper3.Login("tifawe2861@stikezz.com", "123456789T")
// 	//err = scraper3.Login("wuban358369", "123456789T")
// 	err = scraper3.Login("Wade_Leeeee", "923881393time")
// 	if err != nil {
// 		fmt.Println("scraper3", err.Error())
// 		return nil, err
// 	}

// 	fmt.Println("scraper.IsLoggedIn()", scraper.IsLoggedIn())
// 	fmt.Println("scraper3.IsLoggedIn()", scraper3.IsLoggedIn())

// 	userId := "Bitcoin"
// 	next := ""

// 	var backPushPop = list.New()

// 	outStream := make(chan *list.List, 1)

// 	go func() {

// 		//newBegin := true

// 		for _, idxs := range idxss {

// 			newIdxs := make([]FollowerId, 0, len(idxs))

// 			c := true

// 			for {

// 				c = !c

// 				var err error
// 				var users Users

// 				if c {

// 					users, err = FetchFollowers(scraper, userId, next)
// 					if err != nil {
// 						slog.Error("FetchFollowers", err)
// 						time.Sleep(time.Second)
// 						continue
// 					}
// 				} else {
// 					users, err = FetchFollowers(scraper3, userId, next)
// 					if err != nil {
// 						slog.Error("FetchFollowers", err)
// 						time.Sleep(time.Second)
// 						continue
// 					}
// 				}

// 				// users, err := FetchFollowers(scraper, userId, next)
// 				// if err != nil {
// 				// 	slog.Error("FetchFollowers", err)
// 				// 	time.Sleep(time.Second)
// 				// 	continue
// 				// }

// 				fmt.Println("len(users)-----------------", len(users.Profiles))

// 				for k, v := range users.Profiles {
// 					fmt.Println("k", k, "v", v.UserIdAsNumber, v.Username)
// 				}

// 				if len(newIdxs) == 0 {

// 					for k, v := range users.Profiles {
// 						newIdxs = append(newIdxs, FollowerId{
// 							Follower: Follower{
// 								Follower: v.UserIdAsNumber,
// 								UserName: v.Username,
// 							},
// 						})
// 						if k > 5 {
// 							break
// 						}
// 					}
// 				}

// 				profiles, ok := MatchIdx(users, idxs)

// 				fmt.Println("len(profiles)-----------------", len(profiles))

// 				for _, v := range profiles {
// 					backPushPop.PushFront(v)
// 				}

// 				fmt.Printf("!ok %v  users.Next  %s  %d  len(outStream) %d  backPushPop len %d\n", !ok, users.Next, len(users.Next), len(outStream), backPushPop.Len())

// 				time.Sleep(time.Second * 2)

// 				if !ok && len(users.Next) != 0 {
// 					next = users.Next
// 					continue
// 				}

// 				/*
// 					Api has a global limit on how many requests per second are allowed, don’t make requests more than once per 1.5 seconds from one account.
// 					Also each endpoint has its own limits, most of them are 150 requests per 15 minutes.
// 				*/

// 				if backPushPop.Len() == 0 {
// 					next = ""
// 					//time.Sleep(time.Second * 20)
// 					continue
// 				}

// 				idxs = newIdxs
// 				newIdxs = make([]FollowerId, 0, len(idxs))

// 				select {
// 				case <-done:
// 					return
// 				case outStream <- backPushPop:
// 					fmt.Println("case outStream <- backPushPop: ------------")
// 					backPushPop = list.New()
// 					//time.Sleep(time.Second * 20)
// 				}

// 				next = ""
// 				//time.Sleep(time.Second * 20)
// 			}

// 		}

// 	}()

// 	return outStream, nil
// }

// func MatchIdx(data Users, idxs []FollowerId) ([]Profile, bool) {

// 	if len(idxs) == 0 {
// 		return data.Profiles, false
// 	}

// 	for k, v := range data.Profiles {

// 		for _, idx := range idxs {
// 			if v.UserIdAsNumber == idx.Follower.Follower {
// 				return data.Profiles[:k], true
// 			}
// 		}
// 	}

// 	return data.Profiles, false
// }
