package main

import (
	"container/list"
	b64 "encoding/base64"
	"log/slog"
	"strings"

	twitterscraper "github.com/imperatrona/twitter-scraper"
)

/*
users {"Avatar":"https://abs.twimg.com/sticky/default_profile_images/default_profile_normal.png","Banner":"","Biography":"","Birthday":"","FollowersCount":61,"FollowingCount":7,"FriendsCount":77,"IsPrivate":false,"IsVerified":false,"Joined":"2022-06-11T15:16:41Z","LikesCount":7,"ListedCount":0,"Location":"","Name":"Wade","PinnedTweetIDs":[],"TweetsCount":55,"URL":"https://twitter.com/Wade_Leeeee","UserID":"VXNlcjoxNTM1NjQyMTUyNTY2MjU5NzEy","Username":"Wade_Leeeee","Website":"","Sensitive":false,"Following":false,"FollowedBy":false}
k 0 Wade_Leeeee VXNlcjoxNTM1NjQyMTUyNTY2MjU5NzEy
User:1535642152566259712

users {"Avatar":"https://pbs.twimg.com/profile_images/1787913259074691072/XioiWSMF_normal.jpg","Banner":"https://pbs.twimg.com/profile_banners/1568295462486016008/1715106782","Biography":"Positive mind, positive vibes, positive life ✨","Birthday":"","FollowersCount":47,"FollowingCount":0,"FriendsCount":548,"IsPrivate":false,"IsVerified":false,"Joined":"2022-09-09T17:49:17Z","LikesCount":0,"ListedCount":0,"Location":"South Kensington, UK","Name":"Aurora","PinnedTweetIDs":[],"TweetsCount":21,"URL":"https://twitter.com/Shae85083106","UserID":"VXNlcjoxNTY4Mjk1NDYyNDg2MDE2MDA4","Username":"Shae85083106","Website":"","Sensitive":false,"Following":false,"FollowedBy":false}
k 1 Shae85083106 VXNlcjoxNTY4Mjk1NDYyNDg2MDE2MDA4
User:1568295462486016008
*/
type Users struct {
	Profiles []Profile
	Current  string
	Next     string
	Error    error
}

type Profile struct {
	UserIdAsNumber string
	*twitterscraper.Profile
}

type NewTask[T FollowTask | RetweetTask] struct {
	Task T
	Init bool
}

type FollowTask struct {
	// URL    string

	//Init        bool
	TaskTypeStr string
	UserIdStr   string
	UserNameStr string
	Next        string
	Idx         []FollowerId
	NewIdx      []FollowerId
	backPushPop *list.List

	Res Users

	//Scraper *twitterscraper.Scraper
}

func NewFollowTask(userId, userName, taskType string) *FollowTask {

	return &FollowTask{
		backPushPop: list.New(),
		Idx:         make([]FollowerId, 0),
		UserIdStr:   userId,
		UserNameStr: userName,
		TaskTypeStr: taskType,
	}
}

type taskInterface interface {
	TaskType() string
	ID() string
	UserName() string
	Fetch(scraper *twitterscraper.Scraper) error
	InitIdx() bool

	UpdateIdx() (*list.List, bool)
}

// func (f *FollowTask) URl() string {
// 	return f.URL
// }

func (f *FollowTask) InitIdx() bool {

	if len(f.Res.Profiles) > 0 {
		return false
	}

	return true
}

func (f *FollowTask) TaskType() string {
	return f.TaskTypeStr
}

func (f *FollowTask) ID() string {
	return f.UserIdStr
}

func (f *FollowTask) UserName() string {
	return f.UserNameStr
}

func (f *FollowTask) UpdateIdx() (*list.List, bool) {

	//newIdxs := make([]FollowerId, 0, len(f.Idx))

	if len(f.NewIdx) == 0 {
		for k, v := range f.Res.Profiles {
			f.NewIdx = append(f.NewIdx, FollowerId{
				Follower: Follower{
					Follower: v.UserIdAsNumber,
					UserName: v.Username,
				},
			})
			if k > 5 {
				break
			}
		}
	}

	profiles, ok := MatchIdx(f.Res, f.Idx)

	for _, v := range profiles {
		f.backPushPop.PushFront(v)
	}

	slog.Info("MatchIdx", "f.UserIdStr", f.UserIdStr, "f.UserNameStr", f.UserNameStr, "match", ok, "len(profiles)", len(profiles))

	if !ok {
		f.Next = f.Res.Next
		return nil, false
	}

	// 这个想一想
	if f.backPushPop.Len() == 0 {
		f.Next = ""
		//time.Sleep(time.Second * 20)
		return nil, false
	}

	f.Idx = f.NewIdx
	f.NewIdx = make([]FollowerId, 0, len(f.Idx))

	resList := f.backPushPop

	f.backPushPop = list.New()
	return resList, true
}

func MatchIdx(data Users, idxs []FollowerId) ([]Profile, bool) {

	if len(idxs) == 0 {
		return data.Profiles, true
	}

	for k, v := range data.Profiles {

		for _, idx := range idxs {
			if v.UserIdAsNumber == idx.Follower.Follower {
				return data.Profiles[:k], true
			}
		}
	}

	return data.Profiles, false
}

func MatchIdxRetweet(data Users, idxs []RetweeterId) ([]Profile, bool) {

	if len(idxs) == 0 {
		return data.Profiles, true
	}

	for k, v := range data.Profiles {

		for _, idx := range idxs {
			if v.UserIdAsNumber == idx.Retweeter.RetweeterId {
				//if v.UserIdAsNumber == idx.Follower.Follower {
				return data.Profiles[:k], true
			}
		}
	}

	return data.Profiles, false
}

func (f *FollowTask) Fetch(scraper *twitterscraper.Scraper) error {

	// f.UserId = "OnlyDD_D"

	users, newNext, err := scraper.FetchFollowersByUserID(f.UserIdStr, 20, f.Next)

	//users, newNext, err := scraper.FetchFollowers(f.UserId, 20, f.Next)

	if err != nil {
		return err
	}

	usersWithUserNumber := make([]Profile, 0, len(users))

	for _, v := range users {
		sDec, _ := b64.StdEncoding.DecodeString(v.UserID)
		userId, _ := strings.CutPrefix(string(sDec), "User:")
		item := Profile{
			UserIdAsNumber: userId,
			Profile:        v,
		}

		//slog.Info("fetch-----------------", item.UserID, item.Profile.Username, item.Profile.Joined)
		usersWithUserNumber = append(usersWithUserNumber, item)
	}

	res := Users{
		Profiles: usersWithUserNumber,
		Current:  f.Next,
		Next:     newNext,
	}

	slog.Info("follow fetch", "f.UserIdStr", f.UserIdStr, "f.UserNameStr", f.UserNameStr, "res.Current", res.Current, "res.Next", res.Next, "len(res.Profiles)", len(res.Profiles))

	f.Res = res

	return nil

}

type RetweetTask struct {
	//URL     string
	TaskTypeStr string
	TweetId     string
	UserNameStr string
	Next        string
	Idx         []RetweeterId

	//[]RetweeterId
	// backPushPop = list.New()

	backPushPop *list.List
	NewIdx      []RetweeterId

	Res Users
}

// func (r *RetweetTask) URl() string {
// 	return r.URL
// }

func NewRetweetTask(Id, userName, taskType string) *RetweetTask {
	return &RetweetTask{
		TaskTypeStr: taskType,
		TweetId:     Id,
		UserNameStr: userName,
		backPushPop: list.New(),
		Idx:         make([]RetweeterId, 0),
	}
}

func (f *RetweetTask) TaskType() string {
	return f.TaskTypeStr
}

func (f *RetweetTask) UserName() string {
	return f.UserNameStr
}

func (f *RetweetTask) ID() string {
	return f.TweetId
}

// func (f *RetweetTask) SetScraper(scraper *twitterscraper.Scraper) {
// 	f.Scraper = scraper
// }

func (f *RetweetTask) Fetch(scraper *twitterscraper.Scraper) error {

	slog.Info("retweet fetch", "f.TweetId", f.TweetId, "f.UserNameStr", f.UserNameStr, "f.Next", f.Next)

	//return scraper.RetweetsUsers(f.TweetId)

	users, newNext, err := scraper.RetweetsUsers(f.TweetId, 20, f.Next)

	//users, newNext, err := scraper.FetchFollowers(f.UserId, 20, f.Next)

	if err != nil {
		return err
	}

	usersWithUserNumber := make([]Profile, 0, len(users))

	for _, v := range users {
		sDec, _ := b64.StdEncoding.DecodeString(v.UserID)
		userId, _ := strings.CutPrefix(string(sDec), "User:")
		item := Profile{
			UserIdAsNumber: userId,
			Profile:        v,
		}

		//slog.Info("fetch-----------------", item.UserID, item.Profile.Username, item.Profile.Joined)
		usersWithUserNumber = append(usersWithUserNumber, item)
	}

	res := Users{
		Profiles: usersWithUserNumber,
		Current:  f.Next,
		Next:     newNext,
	}

	slog.Info("retweet fetch", "f.TweetId", f.TweetId, "f.UserNameStr", f.UserNameStr, "res.Current", res.Current, "res.Next", res.Next, "len(res.Profiles)", len(res.Profiles))

	f.Res = res

	return nil

}

// func (f *RetweetTask) SetScraperNil() {
// 	f.Scraper = nil
// }

func (f *RetweetTask) InitIdx() bool {

	if len(f.Res.Profiles) > 0 {
		return false
	}

	return true
}

func (f *RetweetTask) UpdateIdx() (*list.List, bool) {

	if len(f.NewIdx) == 0 {
		for k, v := range f.Res.Profiles {
			f.NewIdx = append(f.NewIdx, RetweeterId{
				Retweeter: Retweeter{
					RetweeterId:       v.UserIdAsNumber,
					RetweeterUserName: v.Username,
				},
			})
			if k > 5 {
				break
			}
		}
	}

	profiles, ok := MatchIdxRetweet(f.Res, f.Idx)

	for _, v := range profiles {
		f.backPushPop.PushFront(v)
	}

	slog.Info("MatchIdx", "f.TweetId", f.TweetId, "f.UserNameStr", f.UserNameStr, "match", ok, "len(profiles)", len(profiles))

	if !ok {
		f.Next = f.Res.Next
		return nil, false
	}

	// 这个想一想
	if f.backPushPop.Len() == 0 {
		f.Next = ""
		//time.Sleep(time.Second * 20)
		return nil, false
	}

	f.Idx = f.NewIdx
	f.NewIdx = make([]RetweeterId, 0, len(f.Idx))

	resList := f.backPushPop

	f.backPushPop = list.New()
	return resList, true
}

type TwitterAccount struct {
	User   string
	PassWd string
}

// var users []TwitterAccount = []TwitterAccount{
// 	TwitterAccount{
// 		User:   "Wade_Leeeee",
// 		PassWd: "923881393time",
// 	},
// }
